/*-
 * Copyright (c) 1990 The Regents of the University of California.
 * All rights reserved.
 *
 * This code is derived from software contributed to Berkeley by
 * Chris Torek.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#if !defined(REALLY_HAVE_VSNPRINTF) || defined(MISSING_VSNPRINTF) || defined(BROKEN_VSNPRINTF)

#if defined(LIBC_SCCS) && !defined(lint)
/*static char *sccsid = "from: @(#)snprintf.c	5.1 (Berkeley) 1/20/91";*/
static char *rcsid = "$Id: snprintf.c,v 1.8 1998/08/09 12:21:52 sjg Exp $";
#endif /* LIBC_SCCS and not lint */

#include <stdio.h>
#if defined(sun) && !defined(__svr4__)
# undef _IOSTRG					/* gets it wrong */
#endif
#if __STDC__
#include <stdarg.h>
#else
#include <varargs.h>
#endif
#include <sys/types.h>

#if __STDC__
vsnprintf(char *str, size_t n, char const *fmt, va_list ap)
#else
vsnprintf(str, n, fmt, ap)
	char *str;
	size_t n;
	char *fmt;
	va_list ap;
#endif
{
	int ret;
	FILE f;

#ifdef __SWR
	f._flags = __SWR | __SSTR;
	f._bf._base = f._p = (unsigned char *)str;
	f._bf._size = f._w = n - 1;
	ret = vfprintf(&f, fmt, ap);
	*f._p = 0;
#else
#ifdef _IOSTRG
	f._flag |= _IORW|_IOSTRG;
	f._base = f._ptr = (unsigned char *)str;
#ifdef NO_IOBUF_BUFSIZ
	f._cnt = n - 1;
#else
	f._cnt = f._bufsiz = n - 1;
#endif
	ret = vfprintf(&f, fmt, ap);
	*f._ptr = 0;
#else
	/*
	 * <sjg> can't get more porable than this :-)
	 */
	FILE *fp;
	int fd;
	char tf[24];

	(void) strcpy(tf, "/tmp/.vsnXXXXXXXX");
	if (fd = mkstemp(tf)) {		/* they should have it! */
		unlink(tf); /* its gone, no physical IO should happen */
		if (fp = fdopen(fd, "w+")) {
			ret = vfprintf(fp, fmt, ap);
			if (ret >= n)
				ret = n - 1;
			rewind(fp);
			ret = fread(str, 1, ret, fp);
			str[ret] = '\0';
			fclose(fp);
		}
		close(fd);
	} else
		ret = 0;
#endif
#endif
	return (ret);
}

#if __STDC__
snprintf(char *str, size_t n, char const *fmt, ...)
#else
snprintf(str, n, fmt, va_alist)
	char *str;
	size_t n;
	char *fmt;
	va_dcl
#endif
{
	int ret;
	va_list ap;

	if ((int)n < 1)
		return (EOF);
#if __STDC__
	va_start(ap, fmt);
#else
	va_start(ap);
#endif
	ret = vsnprintf(str, n, fmt, ap);

	va_end(ap);
	return (ret);
}

#ifdef MAIN
int
main()
{
	char buf[128];
	int ret;
	
	ret = snprintf(buf, 8, "arg = %s\n", "it");

	buf[10] = 0;
	printf("%d, '%s'\n", ret, buf);
}
#endif
#endif
