/*
 * NAME:
 *	dh - DH parameters
 *
 * DESCRIPTION:
 *
 * SEE ALSO:
 *	sslfd(3)
 *
 * AUTHOR:
 *	Simon J. Gerraty <sjg@quick.com.au>
 */
/*
 *      @(#)Copyright (c) 1997 Simon J. Gerraty.
 *      
 *      This is free software.  It comes with NO WARRANTY.
 *      Permission to use, modify and distribute this source code 
 *      is granted subject to the following conditions.
 *      1/ that the above copyright notice and this notice 
 *      are preserved in all copies and that due credit be given 
 *      to the author.  
 *      2/ that any changes to this code are clearly commented 
 *      as such so that the author does not get blamed for bugs 
 *      other than his own.
 *      
 *      Please send copies of changes and bug-fixes to:
 *      sjg@quick.com.au
 */
#ifndef lint
static char RCSid[] = "$Id: dh.c,v 1.2 1999/04/23 11:46:53 sjg Exp $";
#endif

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <syslog.h>
#include <string.h>
#include <unistd.h>
#ifndef R_OK
#include <sys/file.h>
#endif

#include "buffer.h"
#include "x509.h"
#include "pem.h"
#include "ssl.h"

#ifdef libsslfd
#include "sslfd.h"
#endif

/*
 * The following was generated by ssleay dh -C from a DH params file
 * generated by ssleay gendh using sound as the source of random bytes.
 */
static unsigned char dh512_p[]={
        0x8C,0x64,0x5B,0x96,0x87,0x14,0x74,0x02,0x88,0x34,0x6C,0x68,
        0x8D,0xC4,0xC7,0x84,0x44,0x10,0x93,0xCB,0x30,0xE2,0x67,0xB9,
        0xF7,0xDD,0xF9,0x6B,0x0A,0x5C,0xF6,0x6B,0x46,0xE9,0xD1,0x6B,
        0x1C,0x88,0xA4,0xA0,0x24,0xC4,0xE0,0xD4,0xB9,0xA3,0xD2,0xBA,
        0x97,0xCB,0x96,0xF1,0x08,0x66,0xBC,0xC6,0x51,0x3C,0x69,0x77,
        0x95,0x00,0x01,0xC3,
};
static unsigned char dh512_g[]={
        0x02,
};

DH *get_dh512()
{
        DH *dh = 0;
#ifndef NO_DH
        if ((dh=DH_new()) == NULL) return(NULL);
        dh->p=BN_bin2bn(dh512_p,sizeof(dh512_p),NULL);
        dh->g=BN_bin2bn(dh512_g,sizeof(dh512_g),NULL);
        if ((dh->p == NULL) || (dh->g == NULL))
                return(NULL);
#endif
        return(dh);
}

void
setDHparams(ctx, file)
	SSL_CTX *ctx;
	const char *file;
{
#ifndef NO_DH
	FILE *fp;
	DH *dh = NULL;

	if (file && *file) {
		if (fp = fopen(file, "r")) {
			dh = PEM_read_DHparams(fp,NULL,NULL);
			fclose(fp);
		}
	}
	if (!dh) {
		dh = get_dh512();
		syslog(LOG_DEBUG, "using default DH params");
	}
	if (dh) {
		SSL_CTX_set_tmp_dh(ctx,dh);
		DH_free(dh);
	}
#endif
}



	
